; ***  C R A S H   ' N   D I V E  ***
;             WIP title ;-)

; Copyright 2004, Thomas Jentzsch
; Version 0.20


;Ideen:
;- verschieden dicke Plattformen. Je dicker desto mehr Geschwindigkeit (quadratisch?)
;  ist fr den Durchbruch ntig und desto mehr bremst sie ab. Ist die Geschwindigkeit
;  zu gering, wird der Ball reflektiert (bounce).
;- jede Dicke bekommt eine eigene Farbe
;- bewegliche Plattformen (pendelnd)
;- mehrfache Plattformen (NUSIZx)
;- Bonus-Plattformen/Items (extra Lifes/Time, nur bei Durchbruch!, ggfs sonst Abzge?)
;- Bouncing gegen Wande bremst (leicht) ab
;- Bouncing von Plattformen bremst strker ab (50%?)
;- Plattformen knnen auch von unten durchbrochen/gebounct werden
;- Scoring:
;  - Restzeit wenn am Boden
;  - zurckgelegte Hhe (Plattformen)
;  - durchbrochene Plattformen (je dicker desto mehr Punkte, quadratisch und mehr?)
;  - von unten durchbrochene Plattformen bringen mehr/weniger Punkte
;- in hheren Leveln:
;  - schmalerer Turm
;  - mehr/dickere/lngere/beweglichere Plattformen
;  - weniger Zeit
;  - andere Schwerkraft/Reibung
;- wenn Zeit um, dann schliet sich (langsam?) der Turm (Toto wird zerquetscht!)
;- Turm wird heller/dunkler nach unten hin
;? keine extra Objekte, evtl. schwarze Fugen in der Mauer

;Technisches:
;x zwei Zufallszahlenvariablen, um auch wieder hochscrollen zu knnen
;x aus dem Bild verschwundene Plattformen werden wiederhergestellt, ggfs. knnte ein
;  kleiner 8-Bit-Puffer dies etwas verzgern
;- Schwerkraft und relative Reibung (Y und X), daraus folgt maximale Geschwindigkeit
;- Animation fr gecrashte Plattformen? (dnner, andere Breite, expandierendes Loch,
;  Farben flashen)
;- Kollisionen werden in Software erkannt


;TODOs:
;x correct pfHeightLst when UP
;+ kernel simplyfied (+~40 bytes!)
;o speeds (gravity, friction)
;o controls:
;  o joystick
;  + paddle
;o collisions (TODO: improve bouncing)
;o sounds
;o platform generation
;o platform disabling (pattern = 0)
;o scoring
;- gameplay
;  - end of level
;  - death (too slow)
;  - new level
;  o wall gradients
;  ? wall color (depending on level)
;? moving platforms
;? heightLst + 3

;Save space:
;- thinner platforms
;- all platforms same width
;+ no lives
;- fixed platform distances

    processor 6502
    include vcs.h


;===============================================================================
; A S S E M B L E R - S W I T C H E S
;===============================================================================

VERSION         = $0020

BASE_ADR        = $f800

DEBUG           = 1
NTSC            = 1

SLIM_SCORE      = 0             ; (+-0)
LIVES           = 0             ; ( -8)
PADDLE          = 1             ; ( -3)


;===============================================================================
; C O N S T A N T S
;===============================================================================

SCW             = 160

DIGIT_HEIGHT    = 9-4

KERNEL_H        = 184+3
TOTO_H          = 12

PF_H            = 5
MAX_LEVEL       = 9

NUM_BLOCKS      = 11

MIN_Y           = TOTO_H  +5 +12         ; minimum top position of Toto
MAX_Y           = KERNEL_H-4 -12         ; maximum bottom position of Toto


  IF NTSC
TOTO_COL        = $1c                   ; yellow
TOTO_COL_STOP   = $3a                   ; orange
TOTO_COL_BACK   = $46                   ; red

SCORE_COLOR     = $0a                   ; white
HISCORE_COLOR   = $46                   ; red
  ELSE
TOTO_COL        = $2e                   ; yellow
TOTO_COL_STOP   = $4a                   ; orange
TOTO_COL_BACK   = $66                   ; red

SCORE_COLOR     = $0a                   ; white
HISCORE_COLOR   = $66                   ; red
  ENDIF


RAND_EOR_8      = $b4                   ; $b2; $e7; $c3
RAND_EOR_8I     = <(RAND_EOR_8 << 1 | 1)
RAND_SEED       = $-1                   ; (unused?)


GRAVITY         = 10-2


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

tmpVars     ds 9

frameCnt    .byte
level       .byte
random      .byte

; platform data:
pfLst       = .                         ; platform data list
xPosLst     ds NUM_BLOCKS               ; platform x-position
pfPatLst    ds NUM_BLOCKS               ; platform pattern
colorLst    ds NUM_BLOCKS               ; platform color
heightLst   ds NUM_BLOCKS               ; total height of platform block
pfHeightLst ds NUM_BLOCKS               ; platform height
pfTypeLst   ds NUM_BLOCKS
;nusizLst    ds NUM_BLOCKS

SIZE_PFLST  = . - pfLst

saveHeight  .byte                       ; saved height of top platform block
saveHeight3 .byte                       ; saveHeight + 3

topRow      .byte                       ; index of top displayed platform
curRow      .byte                       ; index of bottom displayed platform
cxRow       .byte                       ; previous collision row

yTop        ds 2
yTopHi      = yTop
yTopLo      = yTop+1

ySpeed      ds 2
ySpeedHi    = ySpeed
ySpeedLo    = ySpeed+1

xSpeed      ds 2
xSpeedHi    = xSpeed
xSpeedLo    = xSpeed+1

yToto       ds 2
yTotoHi     = yToto
yTotoLo     = yToto+1

xToto       ds 2
xTotoHi     = xToto
xTotoLo     = xToto+1


; wall animation:
wallSwitch  .byte
wallColTop  .byte
wallColBtm  .byte
wallDelay   .byte

  IF LIVES
lives       .byte
  ENDIF

sound       .byte

  IF PADDLE
paddle      .byte
  ENDIF

score       ds 2
scoreHi     = score
scoreLo     = score+1


  ECHO "*** RAM: ", ., " ***"


;===============================================================================
; M A C R O S
;===============================================================================

  MAC DEBUG_BRK
    IF DEBUG
      brk                         ;
    ENDIF
  ENDM

  MAC BIT_B
    .byte   $24
  ENDM

  MAC BIT_W
    .byte   $2c
  ENDM

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM

  MAC CHECKPAGE
    IF >. != >{1}
      ECHO ""
      ECHO "ERROR: different pages! (", {1}, ",", ., ")"
      ECHO ""
      ERR
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================

    SEG     Bank0
    ORG     BASE_ADR

;---------------------------------------------------------------
DrawScreen SUBROUTINE
;---------------------------------------------------------------
.ptrScore       = tmpVars       ;..+7
.tmpY           = tmpVars+8
.blockHeight    = tmpVars+8
;---------------------------------------
.ptrToto    = tmpVars       ;..+1
.ptrObj     = tmpVars+2
.yToto      = tmpVars+4
.tmpColor   = tmpVars+5
.objDelay   = tmpVars+6

    ldy     #1                  ;
    ldx     #4-1                ; 2
.loopScore:
    lda     #>Zero              ; 2
    sta     .ptrScore,x         ; 4
    sta     .ptrScore+4,x       ; 4
    dex                         ; 2
    sty     .tmpY               ; 3
    lda     score,y             ; 4
    pha                         ; 3
    lsr                         ; 2
    lsr                         ; 2
    lsr                         ; 2
    lsr                         ; 2
    tay                         ; 2
    lda     DigitTbl,y          ; 4
    sta     .ptrScore,x         ; 4
    pla                         ; 4
    and     #$0f                ; 2
    tay                         ; 2
    lda     DigitTbl,y          ; 4
    sta     .ptrScore+4,x       ; 4
    ldy     .tmpY               ; 3
    dey                         ; 2
    dex                         ; 2
    bpl     .loopScore          ; 2
;total: 133

.waitTim:
    ldx     INTIM
    bne     .waitTim
;    stx     VBLANK

;===============================================
; some very tricky coding here :-)
    lda     #%10011001
    sta     .blockHeight
    ldy     #DIGIT_HEIGHT
.nextBlock:
    dey
.contBlock:
    sta     WSYNC
;---------------------------------------
    SLEEP   4                   ; 2
    lda     topRow              ; 3         +1 !!!
    sta     curRow              ; 3
    lda     wallColTop          ; 3
    sta     COLUPF              ; 3 = 16
  IF LIVES
    lda     lives               ; 3
    sta     GRP0                ; 3         @22
  ELSE
    SLEEP   3
  ENDIF
    lda     (.ptrScore+4),y     ; 5
    sta     GRP1                ; 3         @30
    lax     (.ptrScore+6),y     ; 5
    lda     (.ptrScore),y       ; 5
    sta     GRP0                ; 3         @43
    lda     (.ptrScore+2),y     ; 5
    sta     GRP0                ; 3         @51
    stx     GRP1                ; 3 = 38    @54
    lsr     .blockHeight        ; 5
    bcs     .nextBlock          ; 2
    bne     .contBlock          ; 2=  9        (9 -> 4 possible)

    sty     GRP1                ; 3
    sty     GRP0                ; 3 =  6    @69

    lda     xToto               ; 3
    jsr     XPosObject0         ; 6 =  9    @02/@09

    lda     #<Toto+TOTO_H-2     ; 2
;    sec
    sbc     yTotoHi             ; 3             C=1!
    sta     .ptrToto            ; 3
    sec                         ; 2
    sbc     #<Toto-KERNEL_H-1   ; 2
    sta     .yToto              ; 3 = 15

    sty     NUSIZ0              ; 3
    sty     COLUP1              ; 2
    dey                         ; 2
    sty     NUSIZ1              ; 3             quad width players
    sta     HMCLR               ; 3 = 14

    lda     #TOTO_COL
    sta     COLUP0

    lda     #$c0                ;               $c0
    sta     PF0
    ldy     level
    lda     PF1Tbl,y
    sta     PF1
;    lda     #$03
;    sta     PF2

    ldy     #KERNEL_H           ; 2
    sta     WSYNC
;---------------------------------------
    bne     .enterKernel        ; 3 =  3

;=====================================================
Kernel:
.waitObject:                    ; 6         @09
    lda     (.ptrScore,x)       ; 6                     waste 6 cycles
    bcs     .skipGraphic        ; 3 =  9    @18

.loopKernel:                     ;          @27
; *** draw platform: ***
    lda     pfHeightLst,x       ; 4
    sta     .objDelay           ; 3
    lda     colorLst,x          ; 4
    sta     .tmpColor           ; 3
    lda     pfPatLst,x          ; 4                     PF-pattern
    sta     GRP1                ; 3
    lda     heightLst,x         ; 4
    tax                         ; 2
    inx                         ; 2 = 29    @56

.loopY0:                        ;           @47 (1st: 56)
  IF PADDLE
    bit     INPT0               ; 4
    bmi     .skipPaddle         ; 2
    sty     paddle              ; 3
.skipPaddle:                    ;   =  7/9
  ENDIF

    lda     #TOTO_H-1           ; 2
    dcp     .yToto              ; 5
    sta     WSYNC               ; 3 = 10    @57 (66)
;---------------------------------------
    bcs     .doDraw1            ; 2
    lda     #0                  ; 2
    BIT_W                       ;-1
.doDraw1:
    lda     (.ptrToto),y        ; 5
    sta     GRP0                ; 3 = 11    @11

    cpx     .objDelay           ; 3
    bcs     .waitObject         ; 2=  5/6

    lda     .tmpColor           ; 3
    and     ColAndTbl-1,x       ; 4
    sta     COLUP1              ; 3 = 10    @26
.skipGraphic:

    dey                         ; 2
    beq     .exitKernel         ; 2=  4    @30

    lda     wallColBtm          ; 3
    cpy     wallSwitch          ; 3
    bcc     .doChange           ; 2
    BIT_W                       ; 1
.doChange:
    sta     COLUPF              ; 3  = 12   @42     (~@48, midline!)

    dex                         ; 2
    bne     .loopY0             ; 2=  4   @46

;*** bottom platform row: ***
    lda     #TOTO_H-1           ; 2
    dcp     .yToto              ; 5
    bcs     .doDraw2            ; 2
    txa                         ; 2                 x=0
    BIT_W                       ;-1
.doDraw2:
    lda     (.ptrToto),y        ; 5 = 15    @61

    SLEEP   3                   ; 3 =  3

; update row counter:
    dec     curRow              ; 5
    dey                         ; 2
    sta     GRP0                ; 3         @74
    beq     .exitKernel         ; 2= 12
;---------------------------------------
; *** no object row, reposition next object: ***
    stx     COLUP1              ; 3         @03     x=0
.enterKernel:
    ldx     curRow              ; 3
    lda     xPosLst,x           ; 4
    sec                         ; 2 =  9    @12
.waitPos:
    sbc     #$0f                ; 2
    bcs     .waitPos            ; 2
    eor     #$07                ; 2
    asl                         ; 2
    asl                         ; 2
    asl                         ; 2
    sta     RESP1               ; 3 = 15    @27..67
    asl                         ; 2
    sta     HMP1                ; 3
    sta     WSYNC               ; 3 =  8    @35..75
;---------------------------------------
    sta     HMOVE               ; 3 =  3

    lda     #TOTO_H-1           ; 2
    dcp     .yToto              ; 5
    bcs     .doDraw0            ; 2
    lda     #0                  ; 2
    BIT_W                       ;-1
.doDraw0:
    lda     (.ptrToto),y        ; 5
    sta     GRP0                ; 3 = 18    @21

    dey                         ; 2
    bne     .loopKernel         ; 2=  4    @25

;  CHECKPAGE Kernel
;---------------------------------------------------------------
.exitKernel:
    sta     WSYNC               ; 3
    sty     PF0
    sty     PF1
    sty     GRP1
;    sty     GRP0
;    sty     PF2
    rts


;---------------------------------------------------------------
Start SUBROUTINE
;---------------------------------------------------------------
    cld                         ;           clear BCD math bit.
    lda     #0
    ldx     #$ff                ;           set stack to beginning.
    txs
.clearLoop:
    tsx
    pha
    bne     .clearLoop


;---------------------------------------------------------------
;GameInit SUBROUTINE
;---------------------------------------------------------------
    inc     random

  IF LIVES
    lda     #$55
    sta     lives
  ENDIF
    lda     #4
    sta     level

    lda     #MAX_Y-40
    sta     yTotoHi
    lda     #SCW/2-2
    sta     xTotoHi

    lda     #$20
    sta     wallColTop
    lda     #$22
    sta     wallColBtm

; not necessary to initialize all blocks:
.loopNew:
    jsr     NewPlatform
    ldx     #6
    cpx     topRow
    bcs     .loopNew

    dex
    stx     topRow
    dex
    stx     cxRow

    jsr     BackupOnly
    sta     yTopHi
    sta     heightLst,x

    inc     curRow

;    lda     #$23
;    sta     scoreHi
    lda     #<VERSION
    sta     scoreLo

MainLoop:
;---------------------------------------------------------------
;VerticalBlank SUBROUTINE
;---------------------------------------------------------------

    lda     #$82
    sta     WSYNC
    sta     VSYNC
  IF PADDLE
    sta     VBLANK
  ENDIF
    sta     WSYNC
    sta     WSYNC
    inc     frameCnt
    lsr
    sta     CTRLPF
    ldx     #44-6
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
  IF PADDLE
    sta     VBLANK
  ENDIF
; VerticalBlank

;---------------------------------------------------------------
;GameCalc SUBROUTINE
;---------------------------------------------------------------
; check for new platform:
    lda     curRow
    bne     .skipNew
    jsr     NewPlatform
.skipNew:

; *** sounds ***
    clc
; make bouncing sound:
    ldx     sound
    beq     .skipSound
    dex
    txa
    ldy     #$0c
    adc     #$13                ;       C=1!
.setAudF0:
    stx     AUDV0
    sty     AUDC0
    sta     AUDF0
    dec     sound
.skipSound:

; *** position score sprites: ***
  IF LIVES
    lda     #53-16+3*6-2        ;               dirty tweak to avoid additional HMCLR
  ELSE
    lda     #53+3*6-5           ;               dirty tweak to avoid additional HMCLR
  ENDIF
    jsr     XPosObject0
    inx
  IF LIVES
    lda     #%011
    sta     NUSIZ0              ;       %011
    stx     NUSIZ1              ;       %001
    lda     #53+8+3*6
  ELSE
    stx     NUSIZ0              ;       %001
    stx     NUSIZ1              ;       %001
    lda     #53+3*6-5+8
  ENDIF
    jsr     XPosObject

    ldx     #$0a
    stx     COLUP0
    stx     COLUP1
; GameCalc

    jsr     DrawScreen

---------------------------------------------------------------
;OverScan SUBROUTINE
;---------------------------------------------------------------
    lda     #36-4
    sta     TIM64T

    lsr     SWCHB
    bcs     .skipReset
    brk
.skipReset:



; *** handle controls ***
  IF PADDLE
    lda     level
    asl
    asl
    sbc     #MAX_LEVEL*4
    adc     #SCW-18
    cmp     paddle
    bcc     .skipLeft
    eor     #$ff
    adc     #SCW-7
    cmp     paddle
    bcs     .skipLeft
    lda     paddle
.skipLeft:
    sta     xTotoHi
  ELSE
    lda     #MAX_LEVEL-1
    sec
    sbc     level
    asl
    asl
    adc     #104
    cmp     xTotoHi
    bit     SWCHA
    bmi     .skipRight
    bcc     .skipLeft
    inc     xTotoHi
.skipRight:
    bvs     .skipLeft
    eor     #$ff
    adc     #SCW-8
    cmp     xTotoHi
    bcs     .skipLeft
    dec     xTotoHi
.skipLeft:
  ENDIF

;; *** <DEBUG> ***
;    ldx     #0
;    ldy     #0
;    lda     SWCHA
;    asl
;    asl
;    bmi     .skipDown
;    ldx     #$80
;    ldy     #-1
;    bmi     .skipUp
;.skipDown:
;    asl
;    bmi     .skipMove
;    ldx     #$80
;    ldy     #0
;.skipUp
;.skipMove:
;    stx     ySpeedLo
;    sty     ySpeedHi
;    tya
;; *** </DEBUG> ***


; *** accellerate Toto vertically: ***
; 1. gravity:(always down)
    ldy     #-1
    lda     #-GRAVITY           ;       y = -1!
    ldx     #ySpeed
    jsr     Add16

; 2: friction (opposite to direction; the faster, the more):
;    asl
;    bcc     .posSpeed
;    iny
;.posSpeed:
; (factor = ySpeed / 16)
; /64: %11000000, 2 x Rol
; /32: %11100000, 3 x Rol
;    lda     ySpeedLo
;    and     #%10000000          ;
;    asl
;    eor     ySpeedHi
;    and     #%10000000
;    eor     ySpeedHi
;    rol
;;    rol
; TODO: correct friction when moving up:
    bpl     .posSpeed
    iny
.posSpeed:                      ;       1/256
    eor     #$ff                ;       a = ySpeedHi
    jsr     Add16               ;       x = #ySpeed

    ldx     ySpeedLo
;    lda     ySpeedHi
    jsr     MoveThem

; *** handle collisions ***
.yBounce    = tmpVars
.width      = tmpVars+1

; *** check for collision ***
; 1. vertical collision?
; calculate collision block:
    ldx     topRow
    lda     #KERNEL_H+3
    sec
    sbc     yTotoHi
.loopSum
    sbc     #3
    bcc     .exitSum
    sbc     heightLst,x
    dex
    bcs     .loopSum
.exitSum:
    adc     pfHeightLst+1,x
    bcs     .incBlock
    adc     #TOTO_H-5
    bcc     .skipInc
.incBlock:
    inx
.skipInc:
    sty     .yBounce            ;       save difference
    cpx     cxRow
    stx     cxRow
    beq     .skipCollision
    bcc     .skipCollision

; 2. horizontal collision?
; TODO?: store width in RAM
    lda     pfPatLst,x
    beq     .skipCollision
    ldy     #1
.loopWidth:
    iny
    asl
    bne     .loopWidth
    tya
    asl
    asl
    sta     .width

    lda     xPosLst,x
    adc     #7+1                ;       C=0!
    sbc     xTotoHi             ;       C=0!
    adc     .width
    bcc     .skipCollision      ;       at least two pixel must overlap!

; *** check for crash speed ***
;    sec
    ldy     pfTypeLst,x
    lda     ySpeedHi
;    bpl     .doPos
;    clc
;    adc     SpeedTbl,y          ;       C=1;
;    bcs     .bounce
;    bcc     .doCrash
;.doPos:
;    sbc     SpeedTbl,y          ;       C=1;
;    bcc     .bounce
;.doCrash:
;    sta     ySpeedHi
    bpl     .skipNeg
    eor     #$ff
.skipNeg:
    sbc     SpeedTbl,y          ;       C=1;
    bcs     .doCrash

; *** bounce from platform ***
.bounce:
    ldy     #0
    lda     #15+1
    bit     ySpeedHi
    bpl     .posSpeed1
    dey
    lda     #-15+1
.posSpeed1:
    ldx     #ySpeed
    jsr     Add16

; reverse direction:
    lda     ySpeedLo
;    clc
;    adc     #1
    eor     #$ff
    sta     ySpeedLo
    tax
    lda     ySpeedHi
    eor     #$ff
;    adc     #0
    sta     ySpeedHi

; move into opposite direction:
;    ldx     ySpeedLo
;    lda     ySpeedHi
    jsr     MoveThem

; start bouncing sound:
    lda     #$0c
    bne     .setSound

; *** crash through platform ***
.doCrash:
; set new, reduced speed:
    bit     ySpeedHi
    bpl     .skipNeg2
    eor     #$ff
.skipNeg2:
    sta     ySpeedHi

    lda     #0
    sta     pfPatLst,x

;TODO: scoring
    ldy     pfTypeLst,x
    lda     ScoreTbl,y
    jsr     Add16_ScorePos

    lda     #0
;TODO: crash sound

.setSound:
    sta     sound

;    lda     colorLst,x
;    eor     #$80
;    sta     colorLst,x
.skipCollision:

.waitTim:
    ldx     INTIM
    bne     .waitTim

    jmp     MainLoop
; OverScan


;---------------------------------------------------------------
MoveThem SUBROUTINE
;---------------------------------------------------------------
; move Toto and platforms
; Input:
; x = speedLo
; a = speedHi
;---------------------------------------
.dY         = tmpVars+1
.tmpSpeed   = tmpVars+2     ; .. 3
.tmpSpeedHi = .tmpSpeed
.tmpSpeedLo = .tmpSpeed+1

    stx     .tmpSpeedLo
    sta     .tmpSpeedHi

; Toto moving up or down?
    ldy     yTotoHi
    cpy     #MAX_Y
    tay
    bmi     .contMove
    ldy     #MIN_Y-1
    cpy     yTotoHi
.contMove:

;    ldx     .tmpSpeedLo
;    lda     .tmpSpeedHi
    bcs     .skipDouble
; 1a. move Toto in opposite direction:
    eor     #$ff                ;       a = .tmpSpeedHi
    tay
    txa
    eor     #$ff
    ldx     #yToto
    jsr     Add16

; 1b. double platform scroll speed:
    lda     .tmpSpeedLo
    asl
    tax
    lda     .tmpSpeedHi
    rol
.skipDouble:
; 2. scroll platforms:
    tay
    txa
    ldx     #yTop
    jsr     Add16

    ldx     topRow
    tay

    sec
    sbc     heightLst,x             ;           calculate difference
    sta     .dY

    tya
    sec
    sbc     saveHeight3
    bcc     .setHeight

    bit     .tmpSpeedHi
    bmi     .moveUp
;---------------------------------------
; platforms moving down:
    tay
; scroll one platform block down:
    inc     topRow
    jsr     RestoreAndBackup
    bcc     .setYTop                ;           height = overflow
    DEBUG_BRK
;---------------------------------------
; platforms moving up:
.moveUp:
; scroll one platform block up:
    dec     topRow
    jsr     RestoreAndBackup
; correct top row value:
    tya
;    clc
    adc     saveHeight3
    tay                             ;           height += 3 - underflow
;---------------------------------------
; set top row value:
.setYTop:
    sty     yTopHi
.setHeight:
    sty     heightLst,x             ;           store new height

; *** move walls: ***
    lda     wallSwitch
    sec
    sbc     .dY
    cmp     #KERNEL_H
    bcc     .skipSwap
    bit     .dY
    bpl     .negSpeed
    sbc     #KERNEL_H
    BIT_W
.negSpeed:
    adc     #KERNEL_H-1

    pha
    lda     wallColTop
    dec     wallDelay
    bpl     .contColor
    ldx     #8
    stx     wallDelay
    adc     #4-1
.contColor:

    ldy     wallColBtm
    sty     wallColTop
    sta     wallColBtm
    pla
.skipSwap:
    sta     wallSwitch
    rts
; MoveThem


;---------------------------------------------------------------
RestoreAndBackup SUBROUTINE
;---------------------------------------------------------------
; restore current height:
    lda     saveHeight          ;       x = current topRow
    sta     heightLst,x
BackupOnly:
; backup new height:
    ldx     topRow
    lda     heightLst,x
    sta     saveHeight
; special value for scrolling up:
    clc
    adc     #3
    sta     saveHeight3
    rts
; RestoreAndBackup


;---------------------------------------------------------------
NewPlatform SUBROUTINE
;---------------------------------------------------------------
; generate new platfrom at bottom
; TODO: better generator

.tmpMaxX    = tmpVars

    inc     cxRow               ; 5
    inc     topRow              ; 5

; *** move platform data:
    ldx     #SIZE_PFLST-1       ; 2
.loopMove:
    lda     pfLst-1,x           ; 4
    sta     pfLst,x             ; 4
    dex                         ; 2
    bne     .loopMove           ; 2= 12/13
;total: 791

; *** create new platform at/below bottom: ***
;---------------------------------------
; 1. platform width (3..8)
    jsr     NextRandom
    and     #$07
    tax
    lda     PlatformTbl,x
    sta     pfPatLst
;---------------------------------------
; 2. platform x-position (also depending on width)
; TODO: evt. MinTbl aus Level berechnen!
    txa
    asl
    asl
    sta     .tmpMaxX
.nextRandom:
    jsr     NextRandom
    and     #$7f
    ldx     level
    clc                         ;           kann ggf. weg
;    lda     #96
;    lda     #32
    adc     MinTbl,x            ;           3..35
    sta     xPosLst             ;           3..130
    adc     .tmpMaxX            ;           0..28
    adc     MinTbl,x
    cmp     #SCW-29             ;           131
    bcs     .nextRandom
;---------------------------------------
; 3. platform type and color:
    jsr     NextRandom
    and     #$03
    tax
    lda     ColorTbl,x
    sta     colorLst

;---------------------------------------
; 4. platform height (1..~20):
    lda     HeightTbl,x
    sta     pfHeightLst

    jsr     NextRandom
    and     #$7f                ;   $7f
    clc
    adc     #40
    sta     heightLst

    stx     pfTypeLst

;; increase score:
;    lda     #1
;    jsr     Add16_ScorePos
    rts
; NewPlatform


;---------------------------------------------------------------
Add16_ScorePos SUBROUTINE
;---------------------------------------------------------------
    ldy     #$00
Add16_Score:
    ldx     #score
    sed
Add16:
    clc
    adc     $01,x
    sta     $01,x
    tya
    adc     $00,x
    sta     $00,x
    cld
Exit:
    rts
; Add16_ScorePos


;---------------------------------------------------------------
NextRandom SUBROUTINE
;---------------------------------------------------------------
    lda     random              ; 3
    lsr                         ; 2
    bcc     .skipEor            ; 2
    eor     #RAND_EOR_8         ; 2
.skipEor:
    sta     random              ; 3
;  IF RANDOM
;    bit     compMode            ; 3
;    bvc     .skipRandom         ; 2
;;    eor     yTotoLo            ; 3
;    eor     frameCnt            ; 3
;.skipRandom:
  ENDIF
    rts
; NextRandom


;---------------------------------------------------------------
XPosObject0 SUBROUTINE
;---------------------------------------------------------------
    ldx     #0              ; 2
XPosObject:
    sec                     ; 2
    sta     WSYNC           ; 3
;---------------------------------------
WaitObject:
    sbc     #$0f            ; 2
    bcs     WaitObject      ; 2

  CHECKPAGE WaitObject

    eor     #$07            ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    sta     HMP0,x          ; 4
    sta.wx  RESP0,x         ; 5     @23!
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    rts
; XPosObject0


CodeEnd:


;===============================================================================
; R O M - T A B L E S
;===============================================================================

    ORG     BASE_ADR + $390

FREE SET . - CodeEnd

DigitTbl:
    .byte   #<Zero,  #<One,   #<Two,   #<Three, #<Four
    .byte   #<Five,  #<Six,   #<Seven, #<Eight, #<Nine

  IF SLIM_SCORE
One:
    .byte   %00000000
    .byte   %00000010
Seven:
    .byte   %00000000
    .byte   %00000010
Four:
    .byte   %00000000
    .byte   %00000010
Zero:
    .byte   %00111100
    .byte   %01000010
    .byte   %00000000
    .byte   %01000010
Three:
    .byte   %00111100
    .byte   %00000010
Nine:
    .byte   %00111100
    .byte   %00000010
Eight:
    .byte   %00111100
    .byte   %01000010
Six:
    .byte   %00111100
    .byte   %01000010
Two:
    .byte   %00111100
    .byte   %01000000
Five:
    .byte   %00111100
    .byte   %00000010
    .byte   %00111100
    .byte   %01000000
    .byte   %00111100
  ELSE
One:
    .byte   %00000000
    .byte   %00001100
Seven:
    .byte   %00000000
    .byte   %00001100
Four:
    .byte   %00000000
    .byte   %00001100
Zero:
    .byte   %01111000
    .byte   %11001100
    .byte   %00000000
    .byte   %11001100
Three:
    .byte   %01111000
    .byte   %00001100
Nine:
    .byte   %01111000
    .byte   %00001100
Eight:
    .byte   %01111000
    .byte   %11001100
Six:
    .byte   %01111000
    .byte   %11001100
Two:
    .byte   %01111000
    .byte   %11000000
Five:
    .byte   %01111000
    .byte   %00001100
    .byte   %01111000
    .byte   %11000000
    .byte   %01111000
  ENDIF

  CHECKPAGE One

ColAndTbl:
  REPEAT 4
    .byte   $f0, $f4, $f8, $fc, $0e
  REPEND


ColorTbl:
    .byte   $de, $fe, $3e, $4e
HeightTbl:
Y SET 0
  REPEAT 4
Y SET Y + PF_H
    .byte   Y+1
  REPEND

SpeedTbl:
    .byte   1, 3, 5, 7
ScoreTbl:
    .byte   $01, $10, $25, $50

PF1Tbl:
    .byte   %00000000
PlatformTbl:
    .byte   %10000000
    .byte   %11000000
    .byte   %11100000
    .byte   %11110000
    .byte   %11111000
    .byte   %11111100
    .byte   %11111110
    .byte   %11111111


;*** level variables: ***
;PF1Tbl:
;    .byte   $00, $80, $c0, $e0, $f0, $f8, $fc, $fe, $ff
MinTbl
Y SET 0
  REPEAT MAX_LEVEL
Y SET Y + 4
    .byte   Y-1
  REPEND

DataEnd:

    ORG BASE_ADR + $3fc - TOTO_H

FREE SET FREE + . - DataEnd

Toto:
    .byte   %00111000
    .byte   %01111100
    .byte   %01111100
    .byte   %11000110
    .byte   %10111010
    .byte   %11111110
    .byte   %11111110
    .byte   %11010110
    .byte   %11010110
    .byte   %01111100
    .byte   %01111100
    .byte   %00111000

    ECHO "*** Free ", FREE, " bytes ***"

    ORG $fffc, 0

    .word   Start
    .word   Start
